require 'observer'
require 'runit/error'
require 'runit/robserver'
require 'runit/testfailure'

module RUNIT
  class TestResult
    include Observable
    attr_reader :run_tests
    def initialize
      @fail = []
      @err = []
      @run_tests = 0
      @run_asserts = 0
    end
  
    def add_failure(at, err)
      @fail.push TestFailure.new(at, err)
      changed
      notify_observers(RObserver::ADD_FAILURE, at, err)
    end
    private :add_failure
  
    def add_error(at, err)
      @err.push TestFailure.new(at, err)
      changed
      notify_observers(RObserver::ADD_ERROR, at, err)
    end
    private :add_error
  
    def update(*arg)
      @run_asserts += 1
    end

    def run_asserts
      @run_asserts
    end
    alias assert_size run_asserts

    def failure_size
      @fail.size
    end

    def error_size
      @err.size
    end

    def failures
      @fail
    end

    def errors
      @err
    end

    def succeed?
      failure_size == 0 && error_size == 0
    end
  
    def start_test(t)
      @run_tests += 1
      changed
      notify_observers(RObserver::START_TEST, t)
    end
    private :start_test
  
    def end_test(t)
      changed
      notify_observers(RObserver::END_TEST, t)
    end
    private :end_test

    def insert_class_name(last_exception_backtrace, test)
      tns = test.type.inspect.split("::")
      last_exception_backtrace[0] += "(" + tns[tns.length-1] + ")"
      last_exception_backtrace
    end
  
    def run(t)
      t.add_observer(self)
      start_test(t)
      begin
        t.run_bare
      rescue AssertionFailedError
        add_failure(insert_class_name($@, t), $!)
      rescue StandardError, ScriptError
        add_error(insert_class_name($@, t), $!)
      end
      end_test(t)
    end
  end
end  
